@@ -26,12 +26,14 @@ mod label;
 mod list;
 mod login_page;
 mod logo;
+mod modal;
 mod nav;
 mod page;
 mod pagesection;
 mod pagesidebar;
 mod popover;
 mod popper;
+mod select;
 mod split;
 mod stack;
 mod switch;
@@ -70,12 +72,14 @@ pub use label::*;
 pub use list::*;
 pub use login_page::*;
 pub use logo::*;
+pub use modal::*;
 pub use nav::*;
 pub use page::*;
 pub use pagesection::*;
 pub use pagesidebar::*;
 pub use popover::*;
 pub use popper::*;
+pub use select::*;
 pub use split::*;
 pub use stack::*;
 pub use switch::*;
